from .abstract import Abstract

class InMemory(object):
    def __init__(self, storage=None):
        self._storage = storage or {}

    def get(self, key):
        if key not in self._storage:
            return None

        return self._storage[key]

    def set(self, key, value):
        self._storage[key] = value

    def remove(self, key):
        if key not in self._storage:
            return

        del self._storage[key]



class DictionaryAccess(object):
    def __getitem__(self, key):
        return self.get(key)

    def __setitem__(self, key, value):
        self.set(key, value)

    def __delitem__(self, key):
        self.remove(key)



import unittest

from .storage import Storage

class StandardTests(unittest.TestCase):
    def get_driver(self):
        return None

    def ready_driver(self):
        pass

    def kill_driver(self):
        pass

    def setUp(self):
        self.driver = self.get_driver()
        self.ready_driver()

        if not self.driver:
            raise unittest.SkipTest('The driver is not defined.')

        self.storage = Storage(self.driver)

    def tearDown(self):
        self.kill_driver()

        del self.storage
        del self.driver

    def test_set_and_get_simple(self):
        self.storage.set('panda', 'something')
        self.assertEqual('something', self.storage.get('panda'))

    def test_set_and_get_document(self):
        expected = {
            'job':      'Detective',
            'location': 'London, UK',
        }

        self.storage.set('holmes', expected)
        self.assertEqual(expected['job'], self.storage.get('holmes')['job'])

    def test_set_twice(self):
        self.storage.set('panda', 'something')
        self.assertEqual('something', self.storage.get('panda'))

        self.storage.set('panda', 'python')
        self.assertEqual('python', self.storage.get('panda'))

    def test_get_nothing(self):
        self.assertEqual(None, self.storage.get('panda'))

    def test_remove_once(self):
        self.storage.set('panda', 'something')
        self.storage.remove('panda')

        self.assertEqual(None, self.storage.get('panda'))

    def test_remove_twice(self):
        self.storage.set('panda', 'something')
        self.storage.remove('panda')
        self.storage.remove('panda') # ensure that there is no error.

        self.assertEqual(None, self.storage.get('panda'))

    def test_dictionary_access(self):
        self.storage['panda'] = 'something'
        self.assertEqual('something', self.storage['panda'])

        del self.storage['panda']
        self.assertEqual(None, self.storage['panda'])



import decimal
import json

from .abstract import Abstract

class DynamoDB(object):
    def __init__(self, storage, namespace = 'default'):
        self._namespace = namespace
        self._storage   = storage

    def get(self, key):
        response = self._table().get_item(Key = {'key': key})

        item = response['Item'] if 'Item' in response else None

        return item['value'] if item else None

    def set(self, key, value):
        self._table().put_item(Item = self._prepare_for_setter({
            'key':   key,
            'value': value,
        }))

    def remove(self, key):
        self._table().delete_item(Key = {'key': key})

    def _table_name(self):
        return 'sandpiper.cache.{}'.format(self._namespace)

    def _table(self):
        return self._storage.Table(self._table_name())

    def _prepare_for_setter(self, data):
        return self._decode(json.dumps(data))

    def _decode(self, data):
        return json.loads(data, parse_float = decimal.Decimal)

    def prepare(self, **kwargs):
        throughput = {
            'ReadCapacityUnits':  kwargs['io_read']  if 'io_read'  in kwargs else 5,
            'WriteCapacityUnits': kwargs['io_write'] if 'io_write' in kwargs else 4,
        }

        self._storage.create_table(
            TableName = self._table_name(),
            KeySchema = [
                {
                    'AttributeName': 'key',
                    'KeyType': 'HASH'
                }
            ],
            AttributeDefinitions = [
                {
                    'AttributeName': 'key',
                    'AttributeType': 'S'
                }
            ],
            ProvisionedThroughput = throughput
        )



class Abstract(object):
    def get(self, key):
        raise NotImplementedError()

    def set(self, key, value):
        raise NotImplementedError()

    def remove(self, key):
        raise NotImplementedError()

    def find(self, pattern):
        raise NotImplementedError()



try:
    from setuptools import setup
except:
    from distutils.core import setup

setup(
    name         = 'sandpiper',
    version      = '0.2.0',
    description  = 'A Generic/Extendible Key-value Store Interface Library',
    license      = 'MIT',
    author       = 'Juti Noppornpitak',
    author_email = 'juti_n@yahoo.co.jp',
    url          = 'https://github.com/shiroyuki/sandpiper',
    packages     = [
        'sandpiper',
        'sandpiper.adapter',
    ],
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries'
    ]
)



from .adapter import InMemory
from .mixin   import DictionaryAccess

class Storage(DictionaryAccess):
    def __init__(self, driver=InMemory()):
        self.driver = driver

    def get(self, key):
        return self.driver.get(key)

    def set(self, key, value):
        self.driver.set(key, value)

    def remove(self, key):
        self.driver.remove(key)

    def find(self, pattern):
        return self.driver.find(pattern)
